#include <stdio.h>

#ifndef _CSTOOLS_H_RAHEN_

#define _CSTOOLS_H_RAHEN_

#define POCF            8

#define ASCII		0
#define ISOLatin1	1
#define ISOLatin2	2
#define CP1250		3
#define Kam		4
#define PCLatin2	5
#define Koi8		6
#define Mac		7

/* makro pro ziskani jmena kodovani ( => nazev programu na prekodovani ) */
#define TRUENAME(index)		cstruename[2*(index)]
#define CODENAME(index)		cstruename[2*(index)+1]
#define ABBRNAME(index)		csabbrev[(index)]

struct tabitem 
{
       char *   name;
       int      index;
};

const struct tabitem itab[] =
{
  { "ASCII",              ASCII       },
  { "ASC",                ASCII       },
  { "ISOLatin1",	  ISOLatin1 },
  { "ISO-Latin-1",        ISOLatin1 },
  { "ISO-8859-1",         ISOLatin1 },
  { "88591",         ISOLatin1 },
  { "ISOLatin2",	  ISOLatin2 },
  { "ISO-Latin-2",        ISOLatin2 },
  { "ISO-8859-2",         ISOLatin2 },
  { "88592",         ISOLatin2 },
  { "IL2",                ISOLatin2 },
  { "Win",		  CP1250      },
  { "Windows",		  CP1250      },
  { "CP1250",		  CP1250      },
  { "Kamenici",           Kam      },
  { "Kam",                Kam      },
  { "KEYBCS2",		  Kam	   },
  { "Latin2",             PCLatin2 },
  { "PCLatin2",           PCLatin2 },
  { "CP852",		  PCLatin2 },
  { "Koi8",               Koi8     },
  { "Koi8CS",             Koi8     },
  { "KOI8-CS",		  Koi8	   },
  { "Mac",                Mac      },
  { "Macintosh",          Mac      },
  { "",                   -1       }
};
/* do tabulky je nutno pridat nazvy pouzivane ve Windows NT */

const char * cstruename[] =
{
  "ASCII",	"ASCII",
  "88591", "ISO-8859-1",
  "88592", "ISO-8859-2",
  "CP1250",	"MS Win",
  "KEYBCS2",	"Kam",
  "CP852",	"PC Latin2",
  "KOI8CS",	"KOI8-CS",
  "MAC",	"Mac",
  "", "",
};

const char * csabbrev[] =
{
  "asc",
  "il1",
  "il2",
  "win",
  "kam",
  "pc2",
  "koi",
  "mac"
};

#define FONTLENGTH              83

const unsigned char * cstab[POCF] =
{
   "AAAAACCCDDEEEEIILLLNNOOOO/RRSSSTTUUUUYZZZaaaaacccddeeeeiilllnnoooo/rrsssttuuuuyzzz",
   "AACCDEELLLNNORRSSSTTUUZZZaaccdeelllnnorrsssttuuzzz",
   "ġţݮ峵",
   "ĥţݎ峾",
   "AAACCDEEILNOSSTUUZZaaaccdeeilnosstuzz",
   "Ǝѐӷב⊙渗ǄЂء墓筘짫",
   "AACCDEEILNO/SSTUZZaaaccdeeilno/sstuzz",
   "ACDEI̅STacdei˗Κst"
};

int whichcode( char * name )
{
   int i;

   if ( name == NULL ) return -1;

   for( i=0; itab[i].index != -1 ; i++ )
     if (strstr( name, itab[i].name ) != NULL 
			|| !stricmp( name, itab[i].name) )
        return itab[i].index;

   return -1;
}

char cstocstab [256];
short int src_coding = -1;
short int dst_coding = -1;

/*
 * inicializuje tabulku pro prevod z kodovani cislo ifrom do kodovani
 * cislo ito . Hodnoty ifrom, ito jsou ziskany pres whichcode()
 * nebo primo dosazenim konstanty (ISOLatin2 ... viz vyse)
 */
int cs_compile( int ifrom, int ito )
{
  int i,j;

  if ( ifrom == -1 || ito == -1 )
	  return -1;

  for(i=0; i<256 ; i++)
	  if ( i<128 ) cstocstab[i] = i;
	  else cstocstab[i] = ' ';

  for(i=0; i<=FONTLENGTH; i++ )
  {
	  j = cstab[ifrom][i];
#ifdef sun
	  if ( j < 0 ) j = 127 - j;
#endif
	  if ( j > 127 ) cstocstab[j] = cstab[ito][i];
  }

  src_coding = ifrom;
  dst_coding = ito;

  return 0;
}


/*
 * prekoduje string what z kodovani ifrom do ito
 * ifrom, ito jsou indexy do tabulky kodu, jsou ziskany
 * pomoci whichcode(), nebo primo pres konstantu (ISOLatin2 ... viz vyse)
 */
int cs_encode(unsigned char * what, int ifrom, int ito )
{
  int i;

  if ( what == NULL ) return -1;
  if ( ifrom == ASCII ) return -1; /* prevod z ASCII nema smysl */
  if ( ifrom == -1 || ito == -1 ) return -1; /* ilegalni kod ? */
  if ( ifrom == ito ) return 0;    /* prevod na stejne kodovani nema smysl */

  if ( ifrom != src_coding || ito != dst_coding )
     cs_compile( ifrom, ito );

  for( ; *what != '\0' ; what++ )
  {

#ifdef sun
     i = (*what<0) ? 127-*what : *what ; 
#else
     i = *what;
#endif

     *what = cstocstab[i];
 
  }
  return 0;
}


#endif
